// BWShow.cpp : implementation file
//

#include "stdafx.h"
#include "BWSim.h"
#include "BWShow.h"
#include "event.h"
#include "EventManager.h"

#include "general.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define HGAP 20
#define HOFFSET 30   //____Left side gap for info____
#define VOFFSET 30	//_____Top Gap for info
#define VGAP 16
#define USEC 10  //__________No of usec per pixel

/////////////////////////////////////////////////////////////////////////////
// CBWShow dialog

extern CBWSim bwsim;
extern CEventManager event_manager;

//....................................................

CBWShow::CBWShow(CWnd* pParent /*=NULL*/)
: CDialog(CBWShow::IDD, pParent)
{

	//{{AFX_DATA_INIT(CBWShow)

	m_hscroll = 0;
	m_vscroll = 0;
	//}}AFX_DATA_INIT

}

//....................................................

void CBWShow::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	//{{AFX_DATA_MAP(CBWShow)

	DDX_Control(pDX, IDC_VSCROLL, m_ctrl_vscroll);
	DDX_Control(pDX, IDC_HSCROLL, m_ctrl_hscroll);
	DDX_Scroll(pDX, IDC_HSCROLL, m_hscroll);
	DDX_Scroll(pDX, IDC_VSCROLL, m_vscroll);
	//}}AFX_DATA_MAP

}

BEGIN_MESSAGE_MAP(CBWShow, CDialog)
	//{{AFX_MSG_MAP(CBWShow)
	ON_WM_VSCROLL()
	ON_WM_PAINT()
	ON_WM_HSCROLL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

	/////////////////////////////////////////////////////////////////////////////
	// CBWShow message handlers

//....................................................

void CBWShow::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{

	// TODO: Add your message handler code here and/or call default

	SCROLLINFO ScrollInfo;
	pScrollBar->GetScrollInfo(&ScrollInfo);  // get information about the scroll

	if(pScrollBar=&m_ctrl_vscroll)
	{
		switch(nSBCode)
		{	
			case SB_BOTTOM:         //Scrolls to the lower right.
				break;

			case SB_ENDSCROLL:      //Ends scroll.
				break;

			case SB_LINEDOWN:       //Scrolls one line down.
				pScrollBar->SetScrollPos(pScrollBar->GetScrollPos() + 1);
				break;

			case SB_LINEUP:         //Scrolls one line up.
				pScrollBar->SetScrollPos(pScrollBar->GetScrollPos() - 1);
				break;

			case SB_PAGEDOWN:       //Scrolls one page down.
				pScrollBar->SetScrollPos(pScrollBar->GetScrollPos() + ScrollInfo.nPage);
				break;

			case SB_PAGEUP:         //Scrolls one page up.
				pScrollBar->SetScrollPos(pScrollBar->GetScrollPos() - ScrollInfo.nPage);
				break;

			case SB_THUMBPOSITION:  //The user has dragged the scroll box (thumb) and released the mouse button. The nPos parameter indicates the position of the scroll box at the end of the drag operation.
				break;

			case SB_THUMBTRACK:     //The user is dragging the scroll box. This message is sent repeatedly until the user releases the mouse button. The nPos parameter indicates the position that the scroll box has been dragged to.
				pScrollBar->SetScrollPos(nPos);
				break;

			case SB_TOP:            //Scrolls to the upper left.
				break;
		}

		UpdateData(true);
		InvalidateRect(m_RectWindow,FALSE);
		UpdateWindow();

	}	

	//CDialog::OnVScroll(nSBCode, nPos, pScrollBar);

}

//....................................................

void CBWShow::OnPaint()
{

	// TODO: Add your message handler code here

	CPaintDC pDC(this);
	CMemDC memDC( &pDC, m_ScrollWindow, m_RectWindow, &m_BufferBackground );
	CDC* dc=memDC.GetSafeCDC();

	// TODO: Add your message handler code here
	//________________Draw the packet window_____________

	int i;

	CPen PenRed(PS_SOLID,1,RGB(255,0,0));
	CBrush BrushRed(RGB(255,0,0));

	CBrush Brush(RGB(0,0,0));


	//____________Set Text attributes_______________

	dc->SelectObject(m_smallfont);
	dc->SetTextColor(RGB(255,0,0));
	dc->SetBkMode(TRANSPARENT);

	//_____Draw dark region

	dc->SelectObject(PenDark);
	dc->SelectObject(BrushDark);
	dc->Rectangle(HOFFSET,VOFFSET,HOFFSET+BW_WIDTH+2,VOFFSET+bwsim.m_nms.total_bandwidth/4);

	//____________Draw Red Band__________

	dc->SelectObject(&PenRed);
	dc->SelectObject(&BrushRed);
	dc->Rectangle(HOFFSET,VOFFSET,HOFFSET+1,VOFFSET+(bwsim.m_nms.m_bwallocated)/4);

	//________Paint the actual bandwidth____

	dc->SelectObject(PenBright);
	dc->SelectObject(BrushBright);



	double xcord,ycord;

	//double stime[MAX_NSRC];

	CEvent* _event;

	//____________Draw bright regions__________

	dc->SelectObject(&PenBright);
	dc->SelectObject(&BrushBright);

	CPen _pen;
	CBrush _brush;

	for(i=0;i<event_manager.m_npastevents;i++)
	{
		_event=event_manager.m_pastlist[i];
		if(_event->m_event_type==PACKET_SEND_END) //_______Time to draw the packet
		{
			ycord=VOFFSET+_event->m_bwlower/4; //_____Datasource no
			xcord=HOFFSET+2+(bwsim.m_currenttime-_event->m_time)/bwsim.m_bwusec;

			//dc->Rectangle(xcord,ycord,xcord+(_event->m_time-stime[_event->m_data])/USEC,ycord+10);

			if(xcord>=(HOFFSET+2))
			{
				_pen.CreatePen(PS_SOLID,1,_event->m_paintcolor);
				_brush.CreateSolidBrush(_event->m_paintcolor);

				dc->SelectObject(_pen);
				dc->SelectObject(_brush);

				
				dc->Rectangle(int(xcord),int(ycord),int(xcord+(_event->m_duration)/bwsim.m_bwusec),int(ycord+(_event->m_bwupper-_event->m_bwlower)/4));
				dc->TextOut(int(xcord),int(ycord),bwsim.itos(_event->m_data+1));

				_pen.DeleteObject();
				_brush.DeleteObject();
				_pen.Detach();
				_brush.Detach();
			}

		}
	}

	//_______Now draw everything

	UpdateData(true);
	memDC.CopyToScreen(0,m_hscroll,m_vscroll);

	// Do not call CDialog::OnPaint() for painting messages

}

//....................................................

BOOL CBWShow::OnInitDialog()
{
	CDialog::OnInitDialog();

	// TODO: Add extra initialization here

	GetDlgItem(IDC_WINDOW)->GetWindowRect(&m_RectWindow);
	ScreenToClient(&m_RectWindow);
	m_RectWindow.DeflateRect(2,2);

	//____Init Pens_____________

	PenDark.CreatePen(PS_SOLID,1,RGB(20,57,5));
	PenBright.CreatePen(PS_SOLID,1,RGB(152,245,115));
	PenClash.CreatePen(PS_SOLID,1,RGB(248,139,50));

	BrushDark.CreateSolidBrush(RGB(20,57,5));
	BrushBright.CreateSolidBrush(RGB(152,245,115));
	BrushClash.CreateSolidBrush(RGB(248,139,50));

	//___________Create display font________

	m_displayfont.CreatePointFont(70,"Arial",NULL);
	m_smallfont.CreatePointFont(70,"Arial",NULL);

	ReInit();

	//_____________Actual channels now = bw/64 ____________

	return TRUE;  // return TRUE unless you set the focus to a control

	// EXCEPTION: OCX Property Pages should return FALSE

}

//....................................................

void CBWShow::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{

	// TODO: Add your message handler code here and/or call default

	SCROLLINFO ScrollInfo;
	pScrollBar->GetScrollInfo(&ScrollInfo);  // get information about the scroll

	if(pScrollBar=&m_ctrl_hscroll)
	{

		switch(nSBCode)
		{
			case SB_BOTTOM:         //Scrolls to the lower right.
				break;

			case SB_ENDSCROLL:      //Ends scroll.
				break;

			case SB_LINEDOWN:       //Scrolls one line down.
				pScrollBar->SetScrollPos(pScrollBar->GetScrollPos() + 1);
				break;

			case SB_LINEUP:         //Scrolls one line up.
				pScrollBar->SetScrollPos(pScrollBar->GetScrollPos() - 1);
				break;

			case SB_PAGEDOWN:       //Scrolls one page down.
				pScrollBar->SetScrollPos(pScrollBar->GetScrollPos() + ScrollInfo.nPage);
				break;

			case SB_PAGEUP:         //Scrolls one page up.
				pScrollBar->SetScrollPos(pScrollBar->GetScrollPos() - ScrollInfo.nPage);
				break;

			case SB_THUMBPOSITION:  //The user has dragged the scroll box (thumb) and released the mouse button. The nPos parameter indicates the position of the scroll box at the end of the drag operation.
				break;

			case SB_THUMBTRACK:     //The user is dragging the scroll box. This message is sent repeatedly until the user releases the mouse button. The nPos parameter indicates the position that the scroll box has been dragged to.
				pScrollBar->SetScrollPos(nPos);
				break;

			case SB_TOP:            //Scrolls to the upper left.
				break;
		}

		UpdateData(true);
		InvalidateRect(m_RectWindow,FALSE);
		UpdateWindow();

	}	

	//CDialog::OnHScroll(nSBCode, nPos, pScrollBar);

}

//....................................................

void CBWShow::ReInit()
{

	short xmax=HOFFSET+BW_WIDTH+2+20;
	short ymax=VOFFSET+bwsim.m_nms.total_bandwidth/4;

	m_ScrollWindow.left=0;
	m_ScrollWindow.top=0;
	m_ScrollWindow.right=xmax;
	m_ScrollWindow.bottom=ymax;

	SCROLLINFO ScrollInfo;
	ScrollInfo.cbSize = sizeof(ScrollInfo);     // size of this structure
	ScrollInfo.fMask = SIF_ALL;                 // parameters to set
	ScrollInfo.nMin = 0;                        // minimum scrolling position
	ScrollInfo.nMax = m_ScrollWindow.bottom-m_RectWindow.Height()+38;                      // maximum scrolling position
	ScrollInfo.nPage = 40;                      // the page size of the scroll box
	ScrollInfo.nPos = 0;                       // initial position of the scroll box
	ScrollInfo.nTrackPos = 0;                   // immediate position of a scroll box that the user is dragging
	m_ctrl_vscroll.SetScrollInfo(&ScrollInfo);

	ScrollInfo.cbSize = sizeof(ScrollInfo);     // size of this structure
	ScrollInfo.fMask = SIF_ALL;                 // parameters to set
	ScrollInfo.nMin = 0;                        // minimum scrolling position
	ScrollInfo.nMax = __max(m_ScrollWindow.right-m_RectWindow.Width()+38,0);       // maximum scrolling position
	ScrollInfo.nPage = 40;                      // the page size of the scroll box
	ScrollInfo.nPos = 0;                       // initial position of the scroll box
	ScrollInfo.nTrackPos = 0;                   // immediate position of a scroll box that the user is dragging
	m_ctrl_hscroll.SetScrollInfo(&ScrollInfo);
}
