// DSAdd.cpp : implementation file
//

#include "stdafx.h"
#include "BWSim.h"
#include "DSAdd.h"

#include "poisson.h"
#include "reserve.h"
#include "datasource.h"

#include "dlgPoisson.h"
#include "dlgProtocol.h"
#include "dlgReservparam.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDSAdd dialog

extern CBWSim bwsim;

//....................................................

CDSAdd::CDSAdd(CWnd* pParent /*=NULL*/)
: CDialog(CDSAdd::IDD, pParent)
{

	//{{AFX_DATA_INIT(CDSAdd)

	m_nsrc = 0;
	m_builtin = FALSE;
	m_defaultprotocol = _T("");
	//}}AFX_DATA_INIT

}

//....................................................

void CDSAdd::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	//{{AFX_DATA_MAP(CDSAdd)

	DDX_Control(pDX, IDC_DEL, m_del);
	DDX_Control(pDX, IDC_DSADD, m_add);
	DDX_Control(pDX, IDC_DSLIST, m_dslist);
	DDX_Control(pDX, IDC_DSTYPES, m_dstypes);
	DDX_Text(pDX, IDC_NSRC, m_nsrc);
	DDV_MinMaxUInt(pDX, m_nsrc, 0, 100);
	DDX_Check(pDX, IDC_PCHECK, m_builtin);
	DDX_Text(pDX, IDC_LABEL, m_defaultprotocol);
	//}}AFX_DATA_MAP

}

BEGIN_MESSAGE_MAP(CDSAdd, CDialog)
	//{{AFX_MSG_MAP(CDSAdd)
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_DSADD, OnDsadd)
	ON_BN_CLICKED(IDC_DEL, OnDel)
	ON_WM_LBUTTONDBLCLK()
	ON_NOTIFY(NM_DBLCLK, IDC_DSLIST, OnDblclkDslist)
	ON_BN_CLICKED(IDC_PROTOCOL, OnProtocol)
	ON_BN_CLICKED(IDC_PCHECK, OnPcheck)
	ON_BN_CLICKED(IDC_DEFAULTPROTOCOL, OnDefaultprotocol)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

	/////////////////////////////////////////////////////////////////////////////
	// CDSAdd message handlers

//....................................................

BOOL CDSAdd::OnInitDialog()
{

	CDialog::OnInitDialog();

	// TODO: Add extra initialization here

	//___________Create Icon Image List

	HICON t;
	m_smallicons.Create(16,16,ILC_COLOR,3,5);
	t=AfxGetApp()->LoadIcon(IDI_POISSON);
	m_smallicons.Add(t);
	m_smallicons.Add(t);
	t=AfxGetApp()->LoadIcon(IDI_AUDIO);
	m_smallicons.Add(t);
	t=AfxGetApp()->LoadIcon(IDI_CBR);
	m_smallicons.Add(t);

	//__________Init DStype list

	m_dstypes.SetImageList(&m_smallicons,LVSIL_SMALL);
	m_dstypes.InsertItem( 0,"Poisson Data Source", 0 );
	m_dstypes.InsertItem( 1,"Reservation Stream", 1 );
	m_dstypes.InsertItem( 2,"Audio Data Source", 2 );
	m_dstypes.InsertItem( 3,"CBR Data Source", 3 );
	m_dstypes.SetItemState(0,LVIS_SELECTED | LVIS_FOCUSED ,LVIS_SELECTED | LVIS_FOCUSED);

	//________Init DS added column list box___________

	m_dslist.SetImageList(&m_smallicons,LVSIL_SMALL);

	m_dslist.InsertColumn(0,"DS type",LVCFMT_LEFT,90,0);
	m_dslist.InsertColumn(1,"DS details",LVCFMT_LEFT,300,1);
	m_dslist.InsertColumn(2,"Slotted",LVCFMT_LEFT,90,2);
	m_dslist.InsertColumn(3,"Protocol",LVCFMT_LEFT,90,3);
	m_dslist.InsertColumn(4,"Serial",LVCFMT_LEFT,40,4);

	m_builtin=false;
	m_defaultprotocol="Default :"+bwsim.m_defaultprotocol;

	UpdateData(false);

	return TRUE;  // return TRUE unless you set the focus to a control

	// EXCEPTION: OCX Property Pages should return FALSE

}

//....................................................

void CDSAdd::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	// TODO: Add your message handler code here

	// Do not call CDialog::OnPaint() for painting messages

}

//....................................................

void CDSAdd::OnDsadd()
{

	// TODO: Add your control notification handler code here
	//___________-Get type

	UpdateData(true);

	int _choice=m_dstypes.GetNextItem(-1,LVNI_ALL | LVNI_SELECTED);
	if(m_nsrc<1 || m_nsrc>100)
	{
		AfxMessageBox("Enter appropriate no. of sources");
		return;
	}

	UINT i;

	if(_choice==-1)
	{
		AfxMessageBox("Please Select a Data Source type to add");
		return;
	}
	else if(_choice==0)
	{
		CDlgPoisson dlgPoisson;

		dlgPoisson.m_packetlength=PACKETSIZE;
		if(dlgPoisson.DoModal()==IDOK)
		{

			for(i=0;i<m_nsrc;i++)
			{
				//__________Make the required data source with the required properties____

				CDataSource* _t;
				if(!dlgPoisson.m_reserve)
				{
					 _t=new CPoisson;
					((CPoisson*)_t)->SetLambda(dlgPoisson.m_lambda);
					((CPoisson*)_t)->m_slotted=dlgPoisson.m_slotted;
					((CPoisson*)_t)->m_packetlength=dlgPoisson.m_packetlength;
				}
				else
				{
					 _t=new CReserve;
					((CReserve*)_t)->SetLambda(dlgPoisson.m_lambda);
					((CReserve*)_t)->m_slotted=dlgPoisson.m_slotted;
					((CReserve*)_t)->m_packetlength=dlgPoisson.m_packetlength;
					
					//_____Reservation Parameters
					((CReserve*)_t)->m_param_reserveinitaldelay=dlgPoisson.dlgReserve.m_sdelay;
					((CReserve*)_t)->m_param_reservepacketprob=dlgPoisson.dlgReserve.m_rprob;
					((CReserve*)_t)->m_param_reservestreamlength=dlgPoisson.dlgReserve.m_slength;
			
				}



				(_t)->m_retranspolicy=(dlgPoisson.m_retrans) ? ALPHA : USUAL;

				_t->type=1;

				//_______Get Size of current array----add at the end
				int _s=m_dslist.GetItemCount();

				//__________Insert in array

				bwsim.AddDataSource(_t);

				//________Insert in GUI__________

				char buffer[3][50];
				
				
				if(!dlgPoisson.m_reserve)
				{

					_gcvt(((CPoisson*)_t)->lambda,5,buffer[0]);
					itoa(_t->m_bw,buffer[1],10);

					CString _ts=(CString("Lambda : ") + CString(buffer[0]) + "  Bandwidth Required : " + CString(buffer[1]));
					m_dslist.InsertItem(LVIF_STATE | LVIF_TEXT | LVIF_IMAGE,_s,"Poisson",0,LVIS_SELECTED | LVIS_FOCUSED,0,_s);
					m_dslist.SetItemText(_s,1,_ts);
					m_dslist.SetItemText(_s,2,((((CPoisson*)_t)->m_slotted)==true) ? "Yes" : "No");
					m_dslist.SetItemText(_s,3,((CPoisson*)_t)->m_protocol);
					m_dslist.SetItemText(_s,4,bwsim.itos(_s));
				}
				else
				{
					_gcvt(((CReserve*)_t)->lambda,5,buffer[0]);
					itoa(_t->m_bw,buffer[1],10);

					CString _ts=(CString("Lambda : ") + CString(buffer[0]) + "  Bandwidth Required : " + CString(buffer[1]));
					m_dslist.InsertItem(LVIF_STATE | LVIF_TEXT | LVIF_IMAGE,_s,"Reserve",0,LVIS_SELECTED | LVIS_FOCUSED,0,_s);
					m_dslist.SetItemText(_s,1,_ts);
					m_dslist.SetItemText(_s,2,((((CReserve*)_t)->m_slotted)==true) ? "Yes" : "No");
					m_dslist.SetItemText(_s,3,((CReserve*)_t)->m_protocol);
					m_dslist.SetItemText(_s,4,bwsim.itos(_s));
				}

			}
		}
	}

/*	else if(_choice==1)
	{
		CDlgReserve dlgReserve;
		dlgReserve.m_packetlength=PACKETSIZE;
		if(dlgReserve.DoModal()==IDOK)
		{

			for(i=0;i<m_nsrc;i++)
			{
				//__________Make the required data source with the required properties____

				CDataSource* _t=new CReserve;
				((CReserve*)_t)->SetLambda(dlgReserve.m_lambda);
				((CReserve*)_t)->m_slotted=dlgReserve.m_slotted;
				((CReserve*)_t)->m_packetlength=dlgReserve.m_packetlength;
				((CReserve*)_t)->m_rpacketprob=dlgReserve.m_rprob;


				(_t)->m_retranspolicy=(dlgReserve.m_retrans) ? ALPHA : USUAL;

				_t->type=1;

				//_______Get Size of current array----add at the end

				int _s=m_dslist.GetItemCount();

				//__________Insert in array

				bwsim.AddDataSource(_t);

				//________Insert in GUI__________

				char buffer[3][50];
				_gcvt(((CReserve*)_t)->lambda,5,buffer[0]);
				itoa(_t->m_bw,buffer[1],10);

				CString _ts=(CString("Lambda : ") + CString(buffer[0]) + "  Bandwidth Required : " + CString(buffer[1]));
				m_dslist.InsertItem(LVIF_STATE | LVIF_TEXT | LVIF_IMAGE,_s,"Reserve",0,LVIS_SELECTED | LVIS_FOCUSED,0,_s);
				m_dslist.SetItemText(_s,1,_ts);
				m_dslist.SetItemText(_s,2,((((CReserve*)_t)->m_slotted)==true) ? "Yes" : "No");
				m_dslist.SetItemText(_s,3,((CReserve*)_t)->m_protocol);
				m_dslist.SetItemText(_s,4,bwsim.itos(_s));

			}
		}
	}*/


	//	bwsim.m_nstations=m_dslist.GetItemCount();

}

//....................................................

void CDSAdd::OnDel()
{

	// TODO: Add your control notification handler code here

	int i=0;

	int _choice=m_dslist.GetNextItem(-1,LVNI_ALL |  LVNI_SELECTED);

	while(_choice!=-1)
	{

		//	char b[100];
		//	itoa(_choice,b,10);
		//	AfxMessageBox(b);

		CDataSource* _t=bwsim.m_ds[_choice];
		delete _t;
		bwsim.m_ds.RemoveAt(_choice,1);
		m_dslist.DeleteItem(_choice);
		UpdateData(false);

		_choice=m_dslist.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);

	}
	bwsim.m_nstations=m_dslist.GetItemCount();

}

//....................................................

void CDSAdd::OnLButtonDblClk(UINT nFlags, CPoint pt)
{

	// TODO: Add your message handler code here and/or call default

	//ScreenToClient(&pt);

	/*
			char buffer[40];
			itoa(pt.x,buffer,10);
			AfxMessageBox(buffer);
			itoa(pt.y,buffer,10);
			AfxMessageBox(buffer);
			
			  
				int i=m_dslist.HitTest(pt,NULL);
				if(i!=-1)
				{
				AfxMessageBox("Clicked on Item");
				}
				
				  
			*/

		CDialog::OnLButtonDblClk(nFlags, pt);
}

//....................................................

void CDSAdd::OnDblclkDslist(NMHDR* pNMHDR, LRESULT* pResult)
{

	// TODO: Add your control notification handler code here

	*pResult = 0;
}

//....................................................

void CDSAdd::OnProtocol()
{

	// TODO: Add your control notification handler code here

	CDlgProtocol dlg_protocol;

	if(dlg_protocol.DoModal()==IDOK)
	{
		int i=0;
		int _choice=m_dslist.GetNextItem(-1,LVNI_ALL |  LVNI_SELECTED);
		while(_choice!=-1)
		{

			//	AfxMessageBox(bwsim.itos(_choice));

			bwsim.m_ds[_choice]->m_protocol=dlg_protocol.m_plist;
			m_dslist.SetItemText(_choice,3,dlg_protocol.m_plist);
			_choice=m_dslist.GetNextItem(_choice, LVNI_ALL | LVNI_SELECTED);

		}
	}
}

//....................................................

void CDSAdd::OnPcheck()
{

	// TODO: Add your control notification handler code here

	UpdateData(true);

}

//....................................................

void CDSAdd::OnDefaultprotocol()
{

	// TODO: Add your control notification handler code here

	CDlgProtocol dlg_protocol;

	if(dlg_protocol.DoModal()==IDOK)
	{
		bwsim.m_defaultprotocol=dlg_protocol.m_plist;
		m_defaultprotocol="Default :"+bwsim.m_defaultprotocol;
		UpdateData(false);

	}

}
