// NMS.cpp: implementation of the CNMS class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "bwsim.h"
#include "NMS.h"
#include "general.h"
#include "error.h"
#include "Dsadd.h"
//#include "Protocol.h"
//#include "Fhop.h"
//#include "VCFhop.h"
#include "tdma.h"
#include "fdma.h"
#include "aloha.h"

#include "Fhop.h"
#include "vcfhop.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

extern CBWSim bwsim;

extern CDSAdd dlg_dsadd;

//....................................................

CNMS::CNMS()
{

	Reset();

	total_bandwidth=MAX_BW;

	//_____Tell the protocols about the main datasource bank
	//tdma.ds_array=m_ds;
	//	fdma.ds_array=m_ds;
	//	aloha.ds_array=m_ds;

}

//....................................................

CNMS::~CNMS()
{

}

//....................................................

int CNMS::AddDataSource(CDataSource * ds)
{
	int i;

	for(i=0;i<MAX_NSRC;i++)
	if(IsAllocated[i]==0)
		break;

	if(i==MAX_NSRC)
		return -1;

	IsAllocated[i]=1;
	m_ds[i]=ds;
	no_ds++;

	//____________Return the data_source identity___

	return i;

}

//....................................................

void CNMS::DeleteDataSource(int nms_id)
{
	if(IsAllocated[nms_id])
	{
		IsAllocated[nms_id]=0;
		no_ds--;
	}

}

//....................................................

void CNMS::Reset()
{
	int i;
	for(i=0;i<MAX_NSRC;i++)
	IsAllocated[i]=0;
	no_ds=0;
	no_protocols=0;
	ClearBW();
}

//....................................................

bool CNMS::AssignProtocols(double current_time)
{
	int bw_counter=0;
	extern CError error;
	int i;
	int ds_count=0;

	//__________Clear previously allocated entries of CProtocol

	for(i=0;i<no_protocols;i++)
	delete m_protocols[i];

	if(dlg_dsadd.m_builtin)
	{
		no_protocols=0;
		CProtocol* current; //_______temp variable___

		//________adding TDMA________

		current=(m_protocols[no_protocols++]=new CTDMA);

		current->Reset();
		current->ds_array=m_ds;
		for(i=0;i<MAX_NSRC;i++)
		{
			if(IsAllocated[i]==1)
			{

				current->AddSource(i);
				IsAllocated[i]=2;
				m_ds[i]->m_pid=no_protocols-1; //_____Set protocol Id of datasource
				ds_count++;
			}
			if(ds_count==10)
				break;
		}
		current->SetBWAvail(64*ds_count);
		current->BWRange(bw_counter,bw_used);

		if(current->Init(current_time)==false)
		{
			error.SetError("TDMA Init Failed");
			return false;
		}

		bw_counter+=current->bw_avail;

		//_____________adding FDMA______________

		ds_count=0;
		current=(m_protocols[no_protocols++]=new CFDMA);
		current->ds_array=m_ds;
		current->Reset();

		for(i=0;i<MAX_NSRC;i++)
		{
			if(IsAllocated[i]==1)
			{

				current->AddSource(i);
				IsAllocated[i]=2;
				m_ds[i]->m_pid=no_protocols-1; //_____Set protocol Id of datasource
				ds_count++;

			}
			if(ds_count==10)
			{
				break;
			}
		}
		current->SetBWAvail(64*ds_count);
		current->BWRange(bw_counter,bw_used);

		if(current->Init(current_time)==false)
		{

			error.SetError("FDMA Init Failed");
			return false;
		}

		bw_counter+=current->bw_avail;

		//_______________Aloha protocol Assignment

		ds_count=0;
		current=(m_protocols[no_protocols++]=new CAloha);
		current->ds_array=m_ds;
		current->Reset();
		for(i=0;i<MAX_NSRC;i++)
		{
			if(IsAllocated[i]==1)
			{
				current->AddSource(i);
				m_ds[i]->m_pid=no_protocols-1; //_____Set protocol Id of datasource
				IsAllocated[i]=2;
				ds_count++;

			}

		}
		current->SetBWAvail(64);
		current->BWRange(bw_counter,bw_used);

		if(current->Init(current_time)==false)
		{

			error.SetError("Aloha Init Failed");
			return false;
		}

		if(ds_count)
			bw_counter+=current->bw_avail;

		//___________Tell how much bw allocated_____

		m_bwallocated=bw_counter;
		return true;
	}
	else //__________User custom selection
		{
		int _tcount=0;
		no_protocols=0;
		CProtocol* current; //_______temp variable___

		//________adding TDMA________

		current=(m_protocols[no_protocols++]=new CTDMA);
		current->Reset();
		current->ds_array=m_ds;
		_tcount=0;

		for(i=0;i<MAX_NSRC;i++)
		{
			if(IsAllocated[i]==1 && m_ds[i]->GetProtocol()=="tdma")
			{

				current->AddSource(i);
				IsAllocated[i]=2;
				m_ds[i]->m_pid=no_protocols-1; //_____Set protocol Id of datasource
				_tcount++;
			}
		}

		if(_tcount>0)
		{
			current->SetBWAvail(current->GetBWRequired());
			current->BWRange(bw_counter,bw_used);

			if(current->Init(current_time)==false)
			{
				error.SetError("TDMA Init Failed");
				return false;
			}

			bw_counter+=current->bw_avail;
			ds_count+=_tcount;
		}

		//_____________adding FDMA______________

		_tcount=0;
		current=(m_protocols[no_protocols++]=new CFDMA);
		current->ds_array=m_ds;
		current->Reset();

		for(i=0;i<MAX_NSRC;i++)
		{
			if(IsAllocated[i]==1 && m_ds[i]->GetProtocol()=="fdma")
			{

				current->AddSource(i);
				IsAllocated[i]=2;
				m_ds[i]->m_pid=no_protocols-1; //_____Set protocol Id of datasource
				_tcount++;

			}
		}

		if(_tcount>0)
		{
			current->SetBWAvail(current->GetBWRequired());
			current->BWRange(bw_counter,bw_used);

			if(current->Init(current_time)==false)
			{

				error.SetError("FDMA Init Failed");
				return false;
			}

			bw_counter+=current->bw_avail;
			ds_count+=_tcount;
		}

		//_______________Aloha protocol Assignment

		_tcount=0;
		current=(m_protocols[no_protocols++]=new CAloha);
		current->ds_array=m_ds;
		current->Reset();
		for(i=0;i<MAX_NSRC;i++)
		{
			if(IsAllocated[i]==1 && m_ds[i]->GetProtocol()=="aloha")
			{
				current->AddSource(i);
				m_ds[i]->m_pid=no_protocols-1; //_____Set protocol Id of datasource
				IsAllocated[i]=2;
				_tcount++;

			}

		}
		if(_tcount>0)
		{
			current->SetBWAvail(current->GetBWRequired());
			current->BWRange(bw_counter,bw_used);

			if(current->Init(current_time)==false)
			{
				error.SetError("Aloha Init Failed");
				return false;
			}
			if(_tcount)
				bw_counter+=current->bw_avail;
			ds_count+=_tcount;
		}

		//	AfxMessageBox("assigned to aloha:" + bwsim.itos(_tcount));

		//_______________FHOP protocol Assignment

		_tcount=0;
		current=(m_protocols[no_protocols++])=(new (CFHOP));
		current->ds_array=m_ds;
		current->Reset();
		for(i=0;i<MAX_NSRC;i++)
		{
			if(IsAllocated[i]==1 && m_ds[i]->GetProtocol()=="fhop")
			{
				current->AddSource(i);
				m_ds[i]->m_pid=no_protocols-1; //_____Set protocol Id of datasource
				IsAllocated[i]=2;
				_tcount++;
			}
		}

		if(_tcount>0)
		{
			current->SetBWAvail(current->GetBWRequired());
			current->BWRange(bw_counter,bw_used);

			if(current->Init(current_time)==false)
			{
				error.SetError("FHOP Init Failed");
				return false;
			}
			if(_tcount)
				bw_counter+=current->bw_avail;
			ds_count+=_tcount;
		}

		//_______________VCFHOP protocol Assignment

		_tcount=0;
		current=(m_protocols[no_protocols++]=new CVCFHOP);
		current->ds_array=m_ds;
		current->Reset();
		for(i=0;i<MAX_NSRC;i++)
		{
			if(IsAllocated[i]==1 && m_ds[i]->GetProtocol()=="vcfhop")
			{
				current->AddSource(i);
				m_ds[i]->m_pid=no_protocols-1; //_____Set protocol Id of datasource
				IsAllocated[i]=2;
				_tcount++;
			}
		}

		if(_tcount>0)
		{
			current->SetBWAvail(current->GetBWRequired());
			current->BWRange(bw_counter,bw_used);

			if(current->Init(current_time)==false)
			{
				error.SetError("VCFHOP Init Failed");
				return false;
			}
			if(_tcount)
				bw_counter+=current->bw_avail;
			ds_count+=_tcount;
		}
		
		
		
		//___________Tell how much bw allocated_____

		m_bwallocated=bw_counter;

		if(ds_count!=no_ds)
		{
			error.SetError("All sources could not be assigned !");
			return false;
		}

		//		AfxMessageBox(bwsim.itos(m_bwallocated));

		return true;
	}

}

//....................................................

void CNMS::Send(double current_time,short pid)
{
	ClearBW();
	if(pid==-1)
	{
		for(int i=0;i<no_protocols;i++)
		m_protocols[i]->Send(current_time);
	}
	else
		m_protocols[pid]->Send(current_time);

}

//....................................................

void CNMS::ClearBW()
{
	int i;
	for(i=0;i<MAX_BW;i++)
	bw_used[i]=-1;
}

//....................................................

CProtocol* CNMS::GetProtocol(CDataSource* ds)
{
	return m_protocols[ds->m_pid];

}
