// Poisson.cpp: implementation of the CPoisson class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "bwsim.h"
#include "Poisson.h"
#include "math.h"
#include "Aloha.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

extern CBWSim bwsim;

//....................................................

CPoisson::CPoisson()
{
	lambda=.1;
	m_slotted=false;
	strcpy(id,"poisson");
	m_bw=64;
	type=1;

	//_______State info__________

	Reset();
}

//....................................................

CPoisson::~CPoisson()
{

}

//....................................................

double CPoisson::GetExpTime()
{
	double t;
	double _u;
	_u=double(rand()+1)/RAND_MAX;
	if(_u>1) _u=1;

	t=-log(_u)/lambda;

	//	double _slottime=bwsim.GetTimeMult(bwsim.m_nms.GetProtocol(this)->bw_avail);

	if(m_slotted)
	{

		//	t=int((t+GetTimeMult()/2)/GetTimeMult())*GetTimeMult();

		t=int((t+m_packetduration/2)/m_packetduration)*m_packetduration;

	}
	return t;

}

//....................................................

void CPoisson::SetLambda(double lamb)
{
	lambda=lamb;

}

//....................................................

void  CPoisson::Reset()
{
	m_packetstarttime=GetExpTime();
	m_packetfinishtime=m_packetstarttime+GetTimeMult();
	m_packetduration=bwsim.GetTimeMult(m_bw,m_packetlength);
	m_retransmission=0;
	m_packetsentat=m_packetfinishtime;

	packets_waiting=1;
	packets_sent=0;
	m_retransmission=0;
	m_retransmissions=0;
	m_dropped=0;
	m_averagedelay=0;
	m_totaldelay=0;
}

//....................................................

bool CPoisson::IsPacketAvail(double current_time)
{
	bool _rval=false;

	//cout << endl << current_time << " -- " << m_packetfinishtime;

	if(packets_waiting>1 || m_packetfinishtime==current_time)
	{
		_rval=true;
	}
	return _rval;
}

//....................................................

void CPoisson::SendPacket()
{
	packets_waiting--;
	packets_sent++;

	if(m_retransmission==0) //___If no packet was dropped___
		m_totaldelay+=bwsim.m_currenttime-bwsim.m_currentevent->m_origintime;
	else
	{
		m_totaldelay+=bwsim.m_currenttime-m_packetsentat;
		m_retransmissions++;
	}	

	m_averagedelay=m_totaldelay/packets_sent;
	m_retransmission=0;
}

//....................................................

bool CPoisson::IsPacketBurst(double current_time)
{

	if(current_time==m_packetfinishtime)
		return true;
	return false;

}

//....................................................

int CPoisson::PacketsWaiting(double current_time)
{
	if(m_packetfinishtime>current_time)
		return packets_waiting-1;
	else
		return
	packets_waiting;
}

//....................................................

void CPoisson::ScheduleNextPacket(double time)
{

	double _wtime;
	if(m_retransmission==0) //________Fresh transmission____
	{
		_wtime=GetExpTime();

		if(_wtime<GetTimeMult())
			_wtime=GetTimeMult();
	}
	else //___________retransmission___________
		{

		_wtime=GetRetransTime(m_retranspolicy);
		if(_wtime<GetTimeMult())
			_wtime=GetTimeMult();
	}

	m_packetfinishtime=time+_wtime;
	m_packetstarttime=m_packetfinishtime-GetTimeMult();
	packets_waiting++;
}

//....................................................

CString CPoisson::GetString()
{
	return "Lambda :" + bwsim.dtos(lambda,7);
}

//....................................................

double CPoisson::GetRetransTime(short method)
{

	if(method==ALPHA && m_slotted) //_______must be aloha
	{
		double alpha=((CAloha*)bwsim.m_nms.GetProtocol(this))->alpha;

		//_______Generate uniformly distributed random number

		double _u;
		_u=double(rand()+1)/RAND_MAX;
		if(_u>1) _u=1;

		int slot=1;

		do
		{
			_u=double(rand()+1)/RAND_MAX;
			if(_u>1) _u=1;

			if(_u>alpha)
				slot++;
			else
				break;
		}
		while(1);

		return m_packetduration*slot;
	}
	else if(method==USUAL)
		return GetExpTime();

	return GetExpTime();
}

//....................................................

double CPoisson::GetTimeMult()
{
	if(!m_slotted)
		return CDataSource::GetTimeMult();
	else
		return m_packetduration;
}
