// Poisson.cpp: implementation of the CReserve class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "bwsim.h"
#include "Reserve.h"
#include "math.h"
#include "Aloha.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

extern CBWSim bwsim;

//....................................................

CReserve::CReserve()
{
	m_slotted=false;
	strcpy(id,"reserve_poisson");
	m_bw=64;
	type=1;
	m_param_reservestreamlength=10;
	m_param_reserveinitaldelay=4;
	m_param_reservepacketprob=0;


	//_______State info__________

	Reset();
}

//....................................................

CReserve::~CReserve()
{

}

//....................................................

double CReserve::GetExpTime()
{
	double t;
	double _u;
	_u=double(rand()+1)/RAND_MAX;
	if(_u>1) _u=1;

	t=-log(_u)/lambda;

	//	double _slottime=bwsim.GetTimeMult(bwsim.m_nms.GetProtocol(this)->bw_avail);

	if(m_slotted)
	{

		//	t=int((t+GetTimeMult()/2)/GetTimeMult())*GetTimeMult();

		t=int((t+m_packetduration/2)/m_packetduration)*m_packetduration;

	}
	return t;

}

//....................................................

void CReserve::SetLambda(double lamb)
{
	lambda=lamb;

}

//....................................................

void  CReserve::Reset()
{
	m_packetstarttime=GetExpTime();
	m_packetfinishtime=m_packetstarttime+GetTimeMult();
	m_packetduration=bwsim.GetTimeMult(m_bw,m_packetlength);
	m_retransmission=0;
	m_packetsentat=m_packetfinishtime;

	packets_waiting=1;
	packets_sent=0;
	m_retransmission=0;
	m_retransmissions=0;
	m_dropped=0;
	m_averagedelay=0;
	m_totaldelay=0;
	m_reservationstream=false;
	m_streamcounter=0;

	int m_data_reservetransmissions=0;


}

//....................................................

bool CReserve::IsPacketAvail(double current_time)
{
	bool _rval=false;

	//cout << endl << current_time << " -- " << m_packetfinishtime;

	if(packets_waiting>1 || m_packetfinishtime==current_time)
	{
		_rval=true;
	}
	return _rval;
}

//....................................................

void CReserve::SendPacket()
{
	packets_waiting--;
	packets_sent++;

	//________Calculate delay___________
	if(m_retransmission==0) //___If no packet was dropped___
		m_totaldelay+=bwsim.m_currenttime-bwsim.m_currentevent->m_origintime;
	else
	{
		m_totaldelay+=bwsim.m_currenttime-m_packetsentat;
		m_retransmissions++;
	}	
	m_averagedelay=m_totaldelay/packets_sent;
	m_retransmission=0;
	
	if(m_reservationstream==false ) //_______Check if sent a reserving packet
	{
		if(m_packetheader & 1)
		{
			m_reservationstream=true;
			m_streamcounter=0;
			//________Turn packetheader info off___
			m_packetheader&=0xfffffffe;
		}
	}
}

//....................................................

bool CReserve::IsPacketBurst(double current_time)
{

	if(current_time==m_packetfinishtime)
		return true;
	return false;

}

//....................................................

int CReserve::PacketsWaiting(double current_time)
{
	if(m_packetfinishtime>current_time)
		return packets_waiting-1;
	else
		return
	packets_waiting;
}

//....................................................

void CReserve::ScheduleNextPacket(double time)
{

	double _wtime;

	//____If already sent the 10th packet reset the reservation packet
	if(m_streamcounter==m_param_reservestreamlength) //_____
	{
		m_data_reservetransmissions++;
		m_reservationstream=false;
		m_streamcounter=0;
	}

	if(m_reservationstream==false)
	{
		if(m_retransmission==0) //________Fresh transmission____
		{
			_wtime=GetExpTime();

			if(_wtime<GetTimeMult())
				_wtime=GetTimeMult();

			double _u=double(rand()+1)/RAND_MAX;
			if(_u>1) _u=1;

			if(_u<m_param_reservepacketprob)
			{
				m_packetheader|=1;
			}

		}
		else //___________retransmission___________
		{

			_wtime=GetRetransTime(m_retranspolicy);
			if(_wtime<GetTimeMult())
				_wtime=GetTimeMult();
		}
		
	}
	else
	{
		//___Schedule in the next slot
		//_______Doesn't care about dropping etc.
		_wtime=GetTimeMult(); 


		//_______Set Waiting Factor after reservation packet is sent___
		if(m_streamcounter==0)
			_wtime=GetTimeMult()*m_param_reserveinitaldelay; //_______

		m_streamcounter++;
	}

	m_packetfinishtime=time+_wtime;
	m_packetstarttime=m_packetfinishtime-GetTimeMult();
	packets_waiting++;
}

//....................................................

CString CReserve::GetString()
{
	return "Lambda :" + bwsim.dtos(lambda,7);
}

//....................................................

double CReserve::GetRetransTime(short method)
{
	if(method==ALPHA && m_slotted) //_______must be aloha
	{
		double alpha=((CAloha*)bwsim.m_nms.GetProtocol(this))->alpha;

		//_______Generate uniformly distributed random number

		double _u;
		_u=double(rand()+1)/RAND_MAX;
		if(_u>1) _u=1;

		int slot=1;

		do
		{
			_u=double(rand()+1)/RAND_MAX;
			if(_u>1) _u=1;

			if(_u>alpha)
				slot++;
			else
				break;
		}
		while(1);
		return m_packetduration*slot;
	}
	else if(method==USUAL)
		return GetExpTime();

	return GetExpTime();
}

//....................................................

double CReserve::GetTimeMult()
{
	if(!m_slotted)
		return CDataSource::GetTimeMult();
	else
		return m_packetduration;
}
