// FHOP.cpp: implementation of the CFHOP class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "FHOP.h"
#include "event.h"
#include "EventManager.h"
#include "BWSim.h"
#include "Poisson.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

extern CBWSim bwsim;
extern CEventManager event_manager;

//....................................................

CFHOP::CFHOP()
{

	m_srcperchannel=2; //________Default

	//______More logic could be there in GetBWRequired()

}

//....................................................

CFHOP::~CFHOP()
{

}

//....................................................

bool CFHOP::Init(double current_time)
{

	int _bwn=0; //_____bandwidth needed__________
	int i=0;
	int _count1=0;

	if(no_ds==0)
		return false;

	//_______Allocate sources to different frequencies

	for(i=0;i<MAX_NSRC;i++)
	{
		if(IsAllocated[i])
		{
			((CPoisson*)ds_array[i])->m_packetduration=bwsim.GetTimeMult(64);
			m_dsbw.SetAt(i,_bwn);
			m_occupied.SetAt(_bwn,false);
			m_clashed.SetAt(_bwn,false);
			m_endtime.SetAt(_bwn,0);
			m_randfreq.SetAt(_count1,_bwn);
			_count1++;

			_bwn+=64;
			if(_bwn>=bw_avail)
			{
				_bwn=0;
				_count1=0;
			}
		}
	}

	m_maxrand=((no_ds)/m_srcperchannel)+((no_ds%m_srcperchannel) ? 0 : -1); //________Base 0

	//AfxMessageBox(bwsim.itos(m_maxrand));

	//	if(bw_avail<_bwn)
	//	{
	//		return false;
	//	}

	return true;
}

//....................................................

void CFHOP::Send(double current_time)
{

	//____________Succesfully sending the packet

	if(bwsim.m_currentevent->m_event_type==PACKET_SEND_END)
	{

		//__________Do statistics part___

		int _id=bwsim.m_currentevent->m_data;
		ds_array[_id]->SendPacket();
		int _freq=bwsim.m_currentevent->m_bwlower;
		m_occupied.SetAt(_freq,false);
		m_clashed.SetAt(_freq,false);
	}

	//_________Check if collision occured before sending

	else if(bwsim.m_currentevent->m_event_type==PACKET_NOTIFY)
	{
		bool _clashed;
		m_clashed.Lookup(bwsim.m_currentevent->m_bwlower,_clashed);
		if(_clashed)
		{
			bwsim.m_ds[bwsim.m_currentevent->m_data]->Dropped(bwsim.m_currentevent->m_origintime);
			return;
		}

		CEvent* _event=new CEvent;
		(*_event)=(*bwsim.m_currentevent);
		_event->m_event_type=PACKET_SEND_END;
		event_manager.AddEvent(_event);
	}
	else if(bwsim.m_currentevent->m_event_type==PACKET_FINISH) //________Schedule this packet for some other time____
	{
		CEvent* _event1=new CEvent;
		CEvent* _event=bwsim.m_currentevent;
		int _id=bwsim.m_ds[_event->m_data]->nms_id;  //_______Its position in IsAllocated Array

		//________Get the freq for this source____

		int freq;
		m_dsbw.Lookup(_id,freq);

		//________Get the endtime___

		double _endtime;
		m_endtime.Lookup(freq,_endtime);

		//________Check if channel is free , Update occupied state___

		if(current_time>_endtime)
		{
			m_occupied.SetAt(freq,false);
			m_clashed.SetAt(freq,false);
		}

		//_____i contains the slot no.

		double _slottime=current_time;
		_event1->m_duration=ds_array[_id]->GetTimeMult();
		_endtime=(_event1->m_time=_slottime+_event1->m_duration-1);
		m_endtime.SetAt(freq,_endtime);

		bool _occupied;
		m_occupied.Lookup(freq,_occupied);

		//	AfxMessageBox("Source No : "+ bwsim.itos(_id)+"\nEndtime : "+bwsim.dtos(_endtime)+
		//		"\nFreq : "+bwsim.itos(freq)+
		//		"\nOccupied : "+((_occupied) ? "Yes" : "No"));

		//________Check if other packets are there for transmission at the same time
		/*	for(int i=0;i<MAX_NSRC;i++)
						{

							if(IsAllocated[i]==1)
							{
					
								int _ofreq;
								m_dsbw.Lookup(i,_ofreq);
								
								if(ds_array[i]->IsPacketBurst(current_time) && freq==_ofreq)
												if(i!=_id)
									{
										_occupied=true;
									}
							}
						}*/

			//	//if(_occupied==true)

		if(_occupied)  //__________ clashed !!! --  now diverge the packets___
		{

			//_____Diverge the two____________

			m_clashed.SetAt(freq,true);

			int i,j;

			//___________Compute random frequencies

			i=int((float(rand())/RAND_MAX)*(m_maxrand+1));
			if(i>m_maxrand)
				i=m_maxrand;

			j=int((float(rand())/RAND_MAX)*(m_maxrand+1));
			if(j>m_maxrand)
				j=m_maxrand;

			int _f1,_f2;
			m_randfreq.Lookup(i,_f1);
			m_randfreq.Lookup(j,_f2);

			//_____Get source id of prev packet

			int _pastid;
			m_pastfreqid.Lookup(freq,_pastid);

			//_____Now set the new frequencies____

			bool _clashed;
			m_clashed.Lookup(freq,_clashed);

			if(!_clashed)
				m_dsbw.SetAt(_pastid,_f1);

			m_dsbw.SetAt(_id,_f2);

			//		AfxMessageBox(bwsim.itos(_pastid)+"  "+bwsim.itos(_f1)+
			//			          "\n"+bwsim.itos(_id)+"  "+bwsim.itos(_f2));

			ds_array[_id]->Dropped(bwsim.m_currentevent->m_origintime); // Inform the ds!!
			m_clashed.SetAt(freq,true);

			delete _event1;
			return;
		}

		//_____No clash yet , first to come !___

		_event1->m_event_type=PACKET_NOTIFY;
		_event1->m_data=_id;
		_event1->m_bwlower=bwind_start+freq;
		_event1->m_bwupper=_event1->m_bwlower+ds_array[_id]->m_transbw;
		_event1->m_origintime=bwsim.m_currentevent->m_origintime;
		event_manager.AddEvent(_event1);
		m_occupied.SetAt(freq,true);

		//______Save the id if this one clashes in future___

		m_pastfreqid.SetAt(freq,_id);
	}
}

//....................................................

void CFHOP::Finish(double current_time)
{

}

//....................................................

int CFHOP::GetBWRequired()
{
	if(m_autobw)
		return 64*((no_ds)/m_srcperchannel)+((no_ds%m_srcperchannel) ? 64 : 0);
	else
		return bw_avail;

}
