// FHOP.cpp: implementation of the CVCFHOP class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VCFHOP.h"
#include "event.h"
#include "EventManager.h"
#include "BWSim.h"
#include "Poisson.h"
#include "Reserve.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

extern CBWSim bwsim;
extern CEventManager event_manager;

//....................................................

CVCFHOP::CVCFHOP()
{

	m_srcperchannel=2; //________Default

	//______More logic could be there in GetBWRequired()

}

//....................................................

CVCFHOP::~CVCFHOP()
{

}

//....................................................

bool CVCFHOP::Init(double current_time)
{

	int _bwn=0; //_____bandwidth needed__________
	int i=0;
	int _count1=0;
	m_maxrand=0;

	if(no_ds==0)
		return false;

	//_____First set at 64 - all
	for(i=0;i<((bw_avail/64)+1);i++)
		m_channelfrequency.SetAt(i,64);

	//m_channelfrequency.SetAt(0,128);

	//_______Allocate sources to different frequencies - 
	//________Deterministic

	//___________count1 represents the channel no - 0 based
	for(i=0;i<MAX_NSRC;i++)
	{
		if(IsAllocated[i])
		{
			m_dsbw.SetAt(i,_bwn);
			m_occupied.SetAt(_bwn,false);
			m_clashed.SetAt(_bwn,false); //_______-Used for first source
			m_endtime.SetAt(_bwn,0);    //________For clashing info____
			m_randfreq.SetAt(_count1,_bwn); //________For hopping channels_____
		

			int _freq;
			m_channelfrequency.Lookup(_count1,_freq);
			m_freqwidth.SetAt(_bwn,_freq);
			m_blockedchannels.SetAt(_bwn,0);

			_bwn+=_freq;

			//_______Set slot time depending on channel bw___
			((CPoisson*)ds_array[i])->m_packetduration=
			bwsim.GetTimeMult(_freq,ds_array[i]->m_packetlength);
//			AfxMessageBox(bwsim.dtos(((CPoisson*)ds_array[i])->m_packetduration));
			((CPoisson*)ds_array[i])->m_transbw=_freq;

			_count1++;


			if(_bwn>=bw_avail) //____End point reached , now recycle____
			{
				m_maxrand=_count1;
				_bwn=0;
				_count1=0;
			}
		}
	}

	m_maxrand--;
	//m_maxrand=((no_ds)/m_srcperchannel)+((no_ds%m_srcperchannel) ? 0 : -1); //________Base 0

	//AfxMessageBox(bwsim.itos(m_maxrand));

	//	if(bw_avail<_bwn)
	//	{
	//		return false;
	//	}

	return true;
}

//....................................................

void CVCFHOP::Send(double current_time)
{

	//____________Succesfully sending the packet

	if(bwsim.m_currentevent->m_event_type==PACKET_SEND_END)
	{

		//__________Do statistics part___

		int _id=bwsim.m_currentevent->m_data;
		ds_array[_id]->SendPacket();
		int _freq=bwsim.m_currentevent->m_bwlower;
		m_occupied.SetAt(_freq,false);
		m_clashed.SetAt(_freq,false);

		//____________Check if packet was reservation packet_____
		if(bwsim.m_currentevent->m_packetheader & 1)
		{
			//Now clear the channel for some time__

			//_____First set the blocking info___
			CReserve* _r=((CReserve*)ds_array[_id]);
			double _blockedtime=(_r->m_param_reserveinitaldelay+_r->m_param_reservestreamlength+1)*(_r->m_packetduration)+bwsim.m_currentevent->m_time;
			m_blockedchannels.SetAt(_freq,_blockedtime);

		//	AfxMessageBox(bwsim.dtos(current_time)+ "  "+bwsim.dtos(_blockedtime));

			//_______Now free the channel for the time being___
			ClearChannel(_freq,_id);
		}
	}

	//_________Check if collision occured before sending

	else if(bwsim.m_currentevent->m_event_type==PACKET_NOTIFY)
	{
		bool _clashed=false;
		m_clashed.Lookup(bwsim.m_currentevent->m_bwlower,_clashed);
		if(_clashed)
		{
			bwsim.m_ds[bwsim.m_currentevent->m_data]->Dropped(bwsim.m_currentevent->m_origintime);
			return;
		}

		CEvent* _event=new CEvent;
		(*_event)=(*bwsim.m_currentevent);
		_event->m_event_type=PACKET_SEND_END;
		event_manager.AddEvent(_event);
	}
	else if(bwsim.m_currentevent->m_event_type==PACKET_FINISH) //________Schedule this packet for some other time____
	{
		CEvent* _event1=new CEvent;
		CEvent* _event=bwsim.m_currentevent;
		int _id=bwsim.m_ds[_event->m_data]->nms_id;  //_______Its position in IsAllocated Array

		//________Get the freq for this source____

		int freq;
		m_dsbw.Lookup(_id,freq);

		//________Get the endtime___

		double _endtime=0;
		m_endtime.Lookup(freq,_endtime);

		//________Check if channel is free , Update occupied state___

		if(current_time>_endtime)
		{
			m_occupied.SetAt(freq,false);
			m_clashed.SetAt(freq,false);
		}

		//_____i contains the slot no.

		double _slottime=current_time;
		int _freqwidth;
		m_freqwidth.Lookup(freq,_freqwidth);

		_event1->m_duration=bwsim.GetTimeMult(ds_array[_id]->m_transbw,ds_array[_id]->m_packetlength);
		_endtime=(_event1->m_time=_slottime+_event1->m_duration-1);
		m_endtime.SetAt(freq,_endtime);

		bool _occupied=false;
		m_occupied.Lookup(freq,_occupied);

		//	AfxMessageBox("Source No : "+ bwsim.itos(_id)+"\nEndtime : "+bwsim.dtos(_endtime)+
		//		"\nFreq : "+bwsim.itos(freq)+
		//		"\nOccupied : "+((_occupied) ? "Yes" : "No"));


		if(_occupied)  //__________ clashed !!! --  now diverge the packets___
		{

			//_____Diverge the two____________

			m_clashed.SetAt(freq,true);

			int i,j;

			//___________Compute random frequencies

		/*	i=int((float(rand())/RAND_MAX)*(m_maxrand+1));
			if(i>m_maxrand)
				i=m_maxrand;

			j=int((float(rand())/RAND_MAX)*(m_maxrand+1));
			if(j>m_maxrand)
				j=m_maxrand;

			int _f1,_f2;
			m_randfreq.Lookup(i,_f1); //_____For pastid 
			m_randfreq.Lookup(j,_f2); //_______for id__________*/


			//_____Get source id of prev packet

			int _pastid;
			m_pastfreqid.Lookup(freq,_pastid);

			//_____Now set the new frequencies____

			bool _clashed;
			m_clashed.Lookup(freq,_clashed);

			//___________Only for the first collision_____
			if(!_clashed) 
			{
				int _f1=GetRandomFreq(current_time);
	
				m_dsbw.SetAt(_pastid,_f1);
				m_freqwidth.Lookup(_f1,_freqwidth);
	
				((CPoisson*)ds_array[_pastid])->m_packetduration=
					bwsim.GetTimeMult(_freqwidth,ds_array[_pastid]->m_packetlength);
				(ds_array[_pastid])->m_transbw=_freqwidth;
				//AfxMessageBox(bwsim.itos(_f1)+"  "+bwsim.itos((ds_array[_pastid])->m_transbw)+"  "+bwsim.itos(_f1));
			}

			int _f2=GetRandomFreq(current_time);
			m_dsbw.SetAt(_id,_f2);
			//_______Set slot time depending on channel bw___
			int _freqwidth;
			m_freqwidth.Lookup(_f2,_freqwidth);

			((CPoisson*)ds_array[_id])->m_packetduration=
				bwsim.GetTimeMult(_freqwidth,ds_array[_id]->m_packetlength);

			(ds_array[_id])->m_transbw=_freqwidth;

			//AfxMessageBox(bwsim.itos(_f2)+"  "+bwsim.itos((ds_array[_id])->m_transbw)+"  "+bwsim.itos(_f2));


			//		AfxMessageBox(bwsim.itos(_pastid)+"  "+bwsim.itos(_f1)+
			//			          "\n"+bwsim.itos(_id)+"  "+bwsim.itos(_f2));

			ds_array[_id]->Dropped(bwsim.m_currentevent->m_origintime); // Inform the ds!!
			m_clashed.SetAt(freq,true);

			delete _event1;
			return;
		}

		//_____No clash yet , first to come !___

		_event1->m_event_type=PACKET_NOTIFY;
		_event1->m_data=_id;
		_event1->m_bwlower=bwind_start+freq;
		_event1->m_bwupper=_event1->m_bwlower+ds_array[_id]->m_transbw;
		_event1->m_origintime=bwsim.m_currentevent->m_origintime;

		_event1->m_packetheader=_event->m_packetheader;
		
		if((bwsim.m_currentevent->m_packetheader & 1) || IsFreqReserved(freq))
			_event1->m_paintcolor=event_manager.m_color_paleyellow;
		event_manager.AddEvent(_event1);
		m_occupied.SetAt(freq,true);

		//______Save the id if this one clashes in future___

		m_pastfreqid.SetAt(freq,_id);
	}
}

//....................................................

void CVCFHOP::Finish(double current_time)
{

}

//....................................................

int CVCFHOP::GetBWRequired()
{
	if(m_autobw)
		return 64*((no_ds)/m_srcperchannel)+((no_ds%m_srcperchannel) ? 64 : 0);
	else
		return bw_avail;

}

int CVCFHOP::GetRandomChannel(double current_time)
{
		double _blocktime=0;
		int i;
		do
		{
		i=int((float(rand())/RAND_MAX)*(m_maxrand+1));
		if(i>m_maxrand)
			i=m_maxrand;

		int _f1;
		m_randfreq.Lookup(i,_f1); 


		_blocktime=0;
		m_blockedchannels.Lookup(_f1,_blocktime);

		}while(_blocktime>current_time);

		return i;

}

int CVCFHOP::GetRandomFreq(double m_currenttime)
{
	double _blocktime=0;
	int _f1;
	int i;

	do
		{
		i=int((float(rand())/RAND_MAX)*(m_maxrand+1));
		if(i>m_maxrand)
			i=m_maxrand;

		m_randfreq.Lookup(i,_f1); //_____For pasti

		_blocktime=0;
		m_blockedchannels.Lookup(_f1,_blocktime);

		}while(_blocktime>bwsim.m_currenttime);

		return _f1;

}



void CVCFHOP::ClearChannel(int freq,int ds)
{
	int i;
	for(i=0;i<MAX_NSRC;i++)
	{
		if(IsAllocated[i])
		{
				int _f;
				m_dsbw.Lookup(i,_f);

				if(_f==freq && i!=ds) //___________Make it hop
				{
					m_dsbw.SetAt(i,GetRandomFreq(bwsim.m_currenttime));
				}
		}
	}


}

bool CVCFHOP::IsFreqReserved(int freq)
{

		double _blocktime=0;
		m_blockedchannels.Lookup(freq,_blocktime);
		if(_blocktime>bwsim.m_currenttime)
		{
			return true;
		}

		return false;
}
